package com.example.gatewayapp;

import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.cloud.client.discovery.EnableDiscoveryClient;
import org.springframework.cloud.gateway.route.RouteLocator;
import org.springframework.cloud.gateway.route.builder.RouteLocatorBuilder;
import org.springframework.context.annotation.Bean;

@SpringBootApplication
@EnableDiscoveryClient
public class GatewayappApplication {

    public static void main(String[] args) {
        SpringApplication.run(GatewayappApplication.class, args);
    }


    /*
    <code>@Bean</code>: This annotation designates the method as a producer of a Spring Bean, which is a reusable component managed by the Spring framework. In this context, the method creates a <code>RouteLocator</code> bean responsible for defining routing rules.

    <code>RouteLocator</code>: This is an interface provided by Spring Cloud Gateway representing a collection of routes. Each route defines how incoming HTTP requests should be handled and where they should be forwarded to.

    <code>routeLocator(RouteLocatorBuilder locatorBuilder)</code>: This method accepts a <code>RouteLocatorBuilder</code> as a parameter. The <code>RouteLocatorBuilder</code> is a utility class in Spring Cloud Gateway, simplifying the creation of routing rules.

    <code>return locatorBuilder.routes().route("user-service-route", route -> route.path("/user/**").uri("lb://USER-SERVICE"))</code>: Here, we use the <code>locatorBuilder</code> to begin defining routing rules. It specifies that a new set of route definitions is being created.

    <code>.build()</code>: This method finalizes the route configuration and constructs the <code>RouteLocator</code> bean.

    In summary, this method configures a route in Spring Cloud Gateway named "user-service-route." It instructs the application to forward incoming requests with paths starting with "/user/" to a service named "USER-SERVICE" using load balancing. The <code>lb://</code> prefix indicates that load balancing should be employed, typically through a service registry like Eureka, to determine the precise service instance to which each request should be dispatched. This routing configuration is particularly valuable in a microservices architecture, where "USER-SERVICE" is one of the registered services, and the Gateway serves as the front-end to route requests based on the URL path.
     */
    @Bean
    public RouteLocator routeLocator(RouteLocatorBuilder locatorBuilder) {
        return locatorBuilder
                .routes()
                .route("user-service-route", route -> route.path("/user/**").uri("lb://USER-SERVICE"))  // http://localhost:9800/user/welcome
                .build();
    }
}
